#ifndef GENERIC_API__SYS_CALL_POOL_HH
#define GENERIC_API__SYS_CALL_POOL_HH

#include <list>

#include "general/mutexlock.hh"
#include "general/Singleton.hh"

namespace GenericAPI
{
  class StatVFSDirect;

  class StatVFSPool
  {
    // Declare the common methods for Singleton class
    SINGLETON_TS_DECL( StatVFSPool );

  public:
    typedef StatVFSDirect info_base_type;
    typedef info_base_type* info_type;
    typedef void (*user_init_cb)( info_type Key );

    static const info_type Base( );

    static void Cleanup( );

    static const info_type StatType( );

    static info_type Request( );
    static void Release( info_type Buffer );
    static void Destroy( info_type Buffer );
    static void UserInitCB( user_init_cb );

  private:
    typedef std::list< info_type > pool_type;

    static pool_type		m_in_use;
    static pool_type		m_available;
    static MutexLock::lock_type	m_pool_baton;
    static user_init_cb		m_user_init_cb;
    static bool			m_initialized;
  };

  inline void StatVFSPool::
  UserInitCB( user_init_cb UserInitCB )
  {
    m_user_init_cb = UserInitCB;
  }
}

#endif /* GENERIC_API__SYS_CALL_POOL_HH */
